unit PlusdbL;

interface
uses PlusLabl, Messages, Classes, Controls, DB, DBTables, dbCtrls, StdCtrls;

type  TDBPlusLabel = class(TPlusLabel)
  private
    FDataLink: TFieldDataLink;
    FAutoDisplay: Boolean;
    FLabelLoaded: Boolean;
    FInDataChange, fInChange: Boolean;
    procedure DataChange(Sender: TObject);
    function GetDataField: string;
    function GetDataSource: TDataSource;
    function GetField: TField;
    procedure SetDataField(const Value: string);
    procedure SetDataSource(Value: TDataSource);
    procedure UpdateData(Sender: TObject);
    function getCaption: string;
  protected
    procedure Change; override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure LoadLabel;
    property Field: TField read GetField;
  published
    property AutoSize default False;
    property DataField: string read GetDataField write SetDataField;
    property DataSource: TDataSource read GetDataSource write SetDataSource;
    end;

procedure Register;

implementation

uses WinTypes, WinProcs, SysUtils{, LibConst};

{ TDBPlusLabel }
const maxtextlen = 32767;

constructor TDBPlusLabel.Create(AOwner: TComponent);
  begin
  inherited Create(AOwner);
  FDataLink := TFieldDataLink.Create;
  FDataLink.OnDataChange := DataChange;
  FDataLink.OnUpdateData := UpdateData;
  AutoSize:= False
  end;

destructor TDBPlusLabel.Destroy;
  begin
  FDataLink.Free;
  FDataLink := nil;
  inherited Destroy;
  end;

function TDBPlusLabel.getCaption: string;
  begin
  Result:= 'DBPlusLabel'
  end;

procedure TDBPlusLabel.Notification(AComponent: TComponent; Operation: TOperation);
  begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and (FDataLink <> nil) and
    (AComponent = DataSource) then DataSource := nil;
  end;


procedure TDBPlusLabel.Change;
  begin
  if fInDataChange then Exit;
  fInChange:= True;
  FDataLink.Edit;
  FDataLink.Modified;
  FLabelLoaded := True;
  inherited Change;
  fInChange:= False
  end;

function TDBPlusLabel.GetDataSource: TDataSource;
  begin
  Result := FDataLink.DataSource;
  end;

procedure TDBPlusLabel.SetDataSource(Value: TDataSource);
  begin
  FDataLink.DataSource := Value;
  end;

function TDBPlusLabel.GetDataField: string;
  begin
  Result := FDataLink.FieldName;
  end;

procedure TDBPlusLabel.SetDataField(const Value: string);
  begin
  FDataLink.FieldName := Value;
  end;

function TDBPlusLabel.GetField: TField;
  begin
  Result := FDataLink.Field;
  end;

{$IFNDEF WIN32}
procedure __AHSHIFT; far; external 'KERNEL' index 113;
{$ENDIF}

procedure TDBPlusLabel.LoadLabel;
  var tmpStream: TMemoryStream;
      pc: PChar; {cp: PChar;}
      i, ilim: Longint;
  begin
  if not FLabelLoaded and (FDataLink.Field is TBlobField) then
    begin
    {Lines.Assign(FDataLink.Field);}
    tmpStream:= TMemoryStream.Create;
    TBlobField(FDataLink.Field).SaveToStream(tmpStream);
    tmpStream.Position:= 0;
    if tmpStream.Size>maxtextlen then ilim:= maxtextlen
                                   else ilim:= tmpStream.Size;
    {$IFNDEF WIN32}      {in 16bit world, #21 char are transformed to #167!!!}
        pc:= tmpStream.Memory;
        for i:= 0 to ilim-1 do
          begin
          if pc^=#167 then pc^:= #21;
          Inc(pc)
          end;
    {$ENDIF}

    pc:= StrAlloc(ilim+1);
    tmpStream.ReadBuffer(pc^, ilim);
    tmpStream.Free;
    pc[ilim]:= #0;
    SetTextBuf(pc);
    StrDispose(pc);
    FLabelLoaded := True;
    end
end;

procedure TDBPlusLabel.DataChange(Sender: TObject);
var tmpstr: string;  i: Integer;
begin
if fInChange then Exit;
fInDataChange:= True;
  {Clear;}
  if FDataLink.Field <> nil then
    if FDataLink.Field is TBlobField then
      begin
          FLabelLoaded := False;
          LoadLabel;
      end

    else begin
         if FDataLink.CanModify then tmpstr := FDataLink.Field.Text
                                else tmpstr := FDataLink.Field.DisplayText;
         {$IFNDEF WIN32}
           for i:= 1 to Length(tmpstr) do
                if tmpstr[i]= #167 then tmpstr[i]:= #21;
         {$ENDIF}
         inherited Caption:= tmpstr;
         FLabelLoaded := True;
         end

  else begin
       if csDesigning in ComponentState then inherited Caption := Name;
       FLabelLoaded := False;
       end;

inherited Change;
fInDataChange:= False;
end;

procedure TDBPlusLabel.UpdateData(Sender: TObject);
  var tmpStream: TStream; tmpstr: string[255]; tlen: Word;  pc: PChar;
  begin
  if FDataLink.Field is TBlobField then
    begin
    tmpStream:= TMemoryStream.Create;
    tlen:= GetTextLen;
    pc:= StrAlloc(tlen+1);
    GetTextBuf(pc, tlen+1);
    tmpStream.WriteBuffer(pc^, tlen);
    tmpStream.Position:= 0;
    TBlobField(FDataLink.Field).LoadFromStream(tmpStream);
    tmpStream.Free;
    StrDispose(pc)
    end

  else begin
       tlen:= GetTextBuf(@tmpstr[1], 255);
       {$IFDEF WIN32} SetLength(tmpstr, tlen);
       {$ELSE}        tmpstr[0]:= Chr(tlen);
       {$ENDIF}
       FDataLink.Field.Text := tmpstr
       end;
end;


procedure Register;
  begin
  RegisterComponents('ECM', [TDBPlusLabel])
  end;


end.
